/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ['AsyncRequest'];
Components.utils.import("resource://hultmann-js-migre-me/util.js");

function AsyncRequest() {
  this.requestUrl = null;
  this.onError = null;
  this.onSuccess = null;
  this.onStart = null;
  this.onStop = null;
}

AsyncRequest.prototype._createXhr = function() {
  var xhr = Components
              .classes['@mozilla.org/xmlextras/xmlhttprequest;1']
              .createInstance(Components.interfaces.nsIXMLHttpRequest);

  xhr.open("GET", this.requestUrl, true);
  xhr.setRequestHeader("Accept-Language", null);
  xhr.setRequestHeader("Cookie", null);
  xhr.setRequestHeader("Accept", "text/xml");
  xhr.setRequestHeader("User-Agent", 'br.mozdev.org/migre.me');
  // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
  //xhr.channel.loadFlags |= Components.interfaces.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;
  return xhr;
};


AsyncRequest.prototype.convert = function(uri) {
  //this.requestUrl = "http://localhost:81/migre.me/api-test.html?url=" + encodeURIComponent(uri.spec);
  this.requestUrl = "http://migre.me/api.xml?url=" + encodeURIComponent(uri.spec);

  var xhr = this._createXhr();
  var me = this;
  xhr.onload = function(evt) {
    me.onStop();
    me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
  }
  xhr.onerror = function(evt) {
    me.onStop();
    me.requestFail(this);
  }
  xhr.send(null);
  this.onStart();
};


AsyncRequest.prototype.restore = function(uri) {
  var idx = uri.path.indexOf('/', 1);
  if (idx > -1) {
    if (uri.path.indexOf('/', idx + 1) == -1) {
      var uri = uri.clone();
      uri.path = uri.path.substr(idx);
    }
  }


  this.requestUrl = "http://migre.me/api_redirect2.xml?url=" + encodeURIComponent(uri.spec);
  var xhr = this._createXhr();
  var me = this;
  xhr.onload = function(evt) {
    me.onStop();
    me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
  }
  xhr.onerror = function(evt) {
    me.onStop();
    me.requestFail(this);
  }
  xhr.send(null);
  this.onStart();
};


AsyncRequest.prototype.requestFail = function(xhr) {
  var errorData = {
    id: 'noconnection',
    xhr: xhr,
    msg: null
  };
  this.onError(errorData);
};


AsyncRequest.prototype.requestSuccess = function(xhr) {
  var errorData = {
    id: 'noconnection',
    xhr: xhr,
    msg: null
  };

  if (xhr.status !== 200) {
    errorData.id = 'invalid';
    this.onError(errorData);
    return;
  }

  var doc = xhr.responseXML;
  if (doc === null) {
    errorData.id = 'invalid';
    this.onError(errorData);
    return;
  }


  if (doc.documentElement.namespaceURI === 'http://www.mozilla.org/newlayout/xml/parsererror.xml') {
    errorData.id = 'malformed';
    this.onError(errorData);
    return;
  }


  var responseData = this.populateResponseData(doc);


  if (this.requestUrl.indexOf("http://migre.me/api.xml") !== 0) {

    if (this.validateUncompressResponse(responseData, errorData)) {
      var uri = util.stringToUri(responseData.url);
      if (uri === null) {
        this.onError({
          id: 'invalid',
          xhr: xhr,
          msg: null
        });
      } else {
        this.onSuccess(uri);
      }
    } else {
      this.onError(errorData);
    }

  } else {

    if (this.validateCompressResponse(responseData, errorData)) {
      var uri = util.stringToUri(responseData.migre);
      if (uri === null) {
        this.onError({
          id: 'invalid',
          xhr: xhr,
          msg: null
        });
      } else {
        this.onSuccess(uri);
      }
    } else {
      this.onError(errorData);
    }
  }
};


AsyncRequest.prototype.populateResponseData = function(doc) {
  var rv = {
    error: null,
    errormessage: null,
    migre: null,
    url: null
  };

  var rvObj  = [      'error',       'errormessage',       'migre',       'url'];
  var xmlObj = ['/item/error', '/item/errormessage', '/item/migre', '/item/url'];


  var type = Components.interfaces.nsIDOMXPathResult.ANY_TYPE;
  var xmlValues = xmlObj.map(function(query) {
    return doc.evaluate(query, doc, null, type, null).iterateNext();
  });

  rvObj.forEach(function(val, idx) {
    if (xmlValues[idx] !== null) {
      rv[val] = xmlValues[idx].textContent;
    }
  });

  return rv;
};


AsyncRequest.prototype.validateCompressResponse = function(responseData, errorData) {
  if (responseData.error === null) {
    errorData.id = 'invalid';
    return false;
  }
  if (responseData.error !== '0') {
    errorData.id = 'couldntconvert';
    errorData.msg = responseData.errormessage === null ? '' : responseData.errormessage;
    return false;
  }

  if (responseData.migre === null) {
    errorData.id = 'invalid';
    return false;
  }

  return true;
};


AsyncRequest.prototype.validateUncompressResponse = function(responseData, errorData) {
  if (responseData.error === null) {
    errorData.id = 'invalid';
    return false;
  }
  if (responseData.error !== '0') {
    errorData.id = 'couldntrestore';
    errorData.msg = responseData.errormessage === null ? '' : responseData.errormessage;
    return false;
  }

  if (responseData.url === null) {
    errorData.id = 'invalid';
    return false;
  }

  return true;
};

