/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ['notificationError', 'util3'];
Components.utils.import("resource://hultmann-js-migre-me/util.js");

function notificationError(errData, browserOrigin) {

  var uri = errData.xhr.channel.originalURI;
  var buildLog = function() {
    var x = errData.xhr;
    return 'GET ' + uri.spec + '\n====================8<-----------------\n' +
           x.status + ' ' + x.statusText + '\n' +
           x.getAllResponseHeaders() + '\n' +
           x.responseText;
  };

  var box = util3.getNotification(browserOrigin);
  var host = uri.host; /*Components
              .classes["@mozilla.org/network/effective-tld-service;1"]
              .getService(Components.interfaces.nsIEffectiveTLDService)
              .getBaseDomain(uri);*/
  var label = util.getText('notification.response.label', host);

  switch (errData.id) {
    case 'couldntconvert':
      showNotification(box, util.getText('notification.request.compress.label', host, errData.msg), null, null);
      break;
    case 'couldntrestore-short':
      showNotification(box, util.getText('notification.request.uncompressShort.label', host), null, null);
      break;
    case 'couldntrestore':
      showNotification(box, util.getText('notification.request.uncompress.label', host, errData.msg), null, null);
      break;
    case 'invalid':
      showNotification(box, label,       util.getText('error.response.details.invalid', host), buildLog());
      break;
    case 'malformed':
      showNotification(box, label,       util.getText('error.response.details.malformed', host), buildLog());
      break;
    case 'noconnection':
      showNotification(box, util.getText('notification.connection.label', host), null, null);
      break;
    //case 1:
    //  showNotification(box, label, util.getText('error.response.details.status'), buildLog());
    //  break;
    //case 'nodata':
    //  showNotification(box, label,       util.getText('error.response.details.nodata'), buildLog());
    //  break;
    default:
      util.myThrow('BUG');
      //break;
  }
}



function showNotification(box, msg, error, txtLog) {
  var chromeWnd = box.ownerDocument.defaultView;

  var buttonConfig = null;
  if (error !== null) {
    buttonConfig = [{
      callback: function(evt) {
        var params = {
          title: util.getText('error.response.details.title'),
          close: util.getText('error.response.details.close.label'),
          label: error,
          description: txtLog
        };
        chromeWnd.openDialog('chrome://migre-me-hultmann/content/warning.xul',
                          '',
                          'chrome,centerscreen,modal',
                          params);
        return true;
      },
      label: util.getText('notification.button.label'),
      accessKey: util.getText('notification.button.accesskey')
    }];
  }

  var box = chromeWnd.gBrowser.getNotificationBox(chromeWnd.gBrowser.selectedBrowser);
  var notif = box.appendNotification(
                  msg,
                  'migre-me-hultmann-warning',
                  'chrome://global/skin/icons/warning-16.png',
                  box.PRIORITY_WARNING_MEDIUM,
                  buttonConfig);
}



const util3 = {

  getNotification: function(browserOrigin) {
    var tabBrowser = browserOrigin.ownerDocument.defaultView.gBrowser;
    var sel = this._findTabByBrowser(browserOrigin, tabBrowser);
    if (sel !== null) {
      tabBrowser.selectedTab = sel;
    } else {
      tabBrowser = Components
        .classes['@mozilla.org/appshell/window-mediator;1']
        .getService(Components.interfaces.nsIWindowMediator)
        .getMostRecentWindow('navigator:browser')
        .gBrowser;
      browserOrigin = tabBrowser.selectedBrowser;
    }
    return tabBrowser.getNotificationBox(browserOrigin);
  },

  _findTabByBrowser: function(browser, tabBrowser) {
      const nodes = tabBrowser.mTabContainer.childNodes;
      for (var idx = 0, len = nodes.length; idx < len; idx++) {
          if (nodes[idx].linkedBrowser === browser) {
              return nodes[idx];
          }
      }
      return null;
  },

  showNotFoundNotification: function(box, newUrl) {
    var buttonConfig = [{
      callback: function(evt) {
        Components
          .classes['@mozilla.org/widget/clipboardhelper;1']
          .getService(Components.interfaces.nsIClipboardHelper)
          .copyString(newUrl);
        return false;
      },
      label: util.getText('notification.node.notfound.button.label'),
      accessKey: util.getText('notification.node.notfound.button.accesskey')
    }];

    var notif = box.appendNotification(
                    util.getText('notification.node.notfound.label', newUrl),
                    'migre-me-hultmann-warning',
                    'chrome://global/skin/icons/warning-16.png',
                    box.PRIORITY_WARNING_MEDIUM,
                    buttonConfig);
  }
};
