/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ['PUBLIC_menuInput', 'PUBLIC_menuTinyUrlLink'];

Components.utils.import("resource://hultmann-js-migre-me/util.js");
Components.utils.import("resource://hultmann-js-migre-me/menu-util.js");

function PUBLIC_menuTinyUrlLink(popupNode, tinyUri) {

  PUBLIC_asyncConfigMenuItems(popupNode, {
    uri: tinyUri,
    typed: null,
    node: null,
    selectionStart: 0,
    selectionEnd: 0
  });

  var doc = popupNode.ownerDocument;

  var cascadeMenu;
  var sepItem = doc.getElementById('hultmann-migre-me-separator');

  if (sepItem) {
    //doc.defaultView.alert('migre.me for Firefox BUG ' + sepItem + menuItem);
    cascadeMenu = doc.getElementById('hultmann-migre-me-cascade');
  } else {
    sepItem = popupNode.insertBefore(doc.createElement('menuseparator'), popupNode.firstChild);
    sepItem.setAttribute('id', 'hultmann-migre-me-separator');
    cascadeMenu = popupNode.insertBefore(doc.createElement('menu'), popupNode.firstChild);
    cascadeMenu.setAttribute('id','hultmann-migre-me-cascade');
  }


  cascadeMenu.value = 'link';
  buildCascadeMenu(cascadeMenu, tinyUri);
}

function buildCascadeMenu(cascadeMenu, uri) {
  cascadeMenu.setAttribute('disabled', 'true');
  cascadeMenu.setAttribute('label', uri.spec);

  if (util2.isOffline() == false) {
    cascadeMenu.setAttribute('class', 'menu-iconic');
    cascadeMenu.setAttribute('image', 'chrome://browser/skin/places/searching_16.png');
  }
}


function PUBLIC_menuInput(popupNode, cmdData, isUrlbar) {
  if (util.isShortHost(cmdData.uri.host)) {
    if (cmdData.uri.path.length === 1) { // '/'
      return;
    }
    PUBLIC_asyncConfigMenuItems(popupNode, cmdData);
  }
  var copy = isUrlbar || cmdData.node.readOnly;
  initInputMenuItems(popupNode, cmdData, copy);
}



function initInputMenuItems(popupNode, cmdData, copy) {
  var doc = popupNode.ownerDocument;

  var cascadeMenu;
  var menuItem;
  var sepItem = doc.getElementById('hultmann-migre-me-separator');

  if (sepItem) {
    menuItem = doc.getElementById('hultmann-migre-me-convert');
    cascadeMenu = doc.getElementById('hultmann-migre-me-cascade');
    doc.defaultView.alert('migre.me for Firefox BUG ' + sepItem + menuItem);
  } else {
    sepItem = popupNode.insertBefore(doc.createElement('menuseparator'), popupNode.firstChild);
    sepItem.setAttribute('id','hultmann-migre-me-separator');
    menuItem = popupNode.insertBefore(doc.createElement('menuitem'), popupNode.firstChild);
    menuItem.setAttribute('id','hultmann-migre-me-convert');
    cascadeMenu = popupNode.insertBefore(doc.createElement('menu'), popupNode.firstChild);
    cascadeMenu.setAttribute('id','hultmann-migre-me-cascade');
  }


  if (util.isShortHost(cmdData.uri.host)) {
    menuItem.hidden = true;
    cascadeMenu.value = copy ? 'inputReadOnly' : 'inputEdit';
    buildCascadeMenu(cascadeMenu, cmdData.uri);
  } else {
    cascadeMenu.hidden = true;
    toTinyUrlCmd(cmdData, menuItem, copy);
  }
}



function toTinyUrlCmd(cmdData, menuItem, copy) {
  var browser = menuItem.ownerDocument.defaultView.gBrowser.selectedBrowser;
  var menuUrl = util2.formatUrl.crop(cmdData.uri);
  var onCommand;
  var label;
  var ak;

  if (copy) {
    onCommand = function(evt) {
      var onSuccess = function(newUri) {
        util2.toClipboard(newUri.spec);
      };
      util2.toClipboard('');
      util2.toggleUrl(cmdData.uri, browser, onSuccess);
    };
    label = util.getText('context.copyCompressed.label', 'migre.me', menuUrl.item);
    ak = util.getText('context.copyCompressed.accesskey');

  } else {
    onCommand = function(evt) {
      var onSuccess = function(newUri) {
        Components.utils.import("resource://hultmann-js-migre-me/run-input.js");
        PUBLIC_replaceCore(cmdData, newUri, browser);
      };
      util2.toggleUrl(cmdData.uri, browser, onSuccess);
    };
    label = util.getText('context.replaceCompressed.label', 'migre.me', menuUrl.item);
    ak = util.getText('context.replaceCompressed.accesskey');
  }

  menuItem.setAttribute('label', label);
  menuItem.setAttribute('accesskey', ak);


  menuItem.setAttribute('image', util2.getFavicon(cmdData.uri));
  menuItem.setAttribute('class', 'menuitem-iconic');


  if (util2.isOffline()) {
    menuItem.setAttribute('disabled', 'true');
  } else {
    //var title = util2.getTitleFromHistory(cmdData.uri);
    //if (title) {
    //  menuItem.setAttribute('tooltiptext', title + '\n' + menuUrl.tooltip);
    //} else {
      menuItem.setAttribute('tooltiptext', menuUrl.tooltip);
    //}
    menuItem.addEventListener('command', onCommand, false);
  }

}
